package sharin.unlinq.iterable;

import java.util.Iterator;
import java.util.Queue;

import sharin.unlinq.Func;
import sharin.unlinq.QueuedIterator;

public class TakeWhileIterable<T> implements Iterable<T> {

    private final Iterable<T> iterable;

    private final Func<T, Boolean> predicate;

    public TakeWhileIterable(Iterable<T> iterable, Func<T, Boolean> predicate) {
        this.iterable = iterable;
        this.predicate = predicate;
    }

    public Iterator<T> iterator() {
        return new QueuedIterator<T, T, T>(iterable.iterator()) {

            private boolean done;

            @Override
            protected void addElement(Queue<T> queue, T t) {

                if (done) {
                    return;
                }

                if (!predicate.call(t)) {
                    done = true;
                    return;
                }

                queue.add(t);
            }

            @Override
            protected T toResult(T e) {
                return e;
            }
        };
    }
}
